#!/bin/sh
#
# This script installs the MacFile application and associated files to your Haiku system.
#
# If you tried to install by opening this script from the tracker and it opened in a text editor
# instead of running, then try the following:
#
#	1. Open a terminal window
#	2. In the terminal, navigate to the directory where this script is located
#	3. Type in the following command and hit return
#
#			chmod guoa+x install-macfile.sh
#
#	4. Close the terminal window
#	5. Double click on install-macfile.sh from the tracker again
#
# NOTE: This script performs a "clean install", so if your installation becomes corrupted,
#       you can re-run this script to get everything back to normal (note that your settings
#       are preserved).
#

cd $(dirname "$0")

LINK_PREF_DIRECTORY=/boot/home/config/be/Preferences
LINK_APPL_DIRECTORY=/boot/home/config/be/Applications
BIN_DIRECTORY=/boot/home/config/bin

ARCHIVEDIR=`dirname "$0"`
ARCHIVE="install.zip"

#name for the (deprecated) uam volume folder and files
UAM_FOLDER="BeUAM_Volume"
UAM_DIRECTORY=/boot/home/BeUAM_Volume
UAM_ARCHIVE="SG-UAMPackage.pkg.zip"

#name for new style mac utility share
MACUTILITY_VOLUME="/boot/home/MacUtilityVolume"

#name for create share add-on
CREATESHARE_ADDON="CreateAfpShare"
TRACKER_ADDON_DIRECTORY="/boot/home/config/add-ons/Tracker"

#name for the apple script that enables clear text passwords
CLRTXT_SCRIPT_NAME="cleartext.scpt"

#name for the read me file located on the mac utilities volume
README_NAME="ReadMe.rtf"

start()
{ 
	if [ -f $1 ]; then 
	        $1 $2 & 
	else 
	        echo There is no $1 
	fi 
}

function installFiles {
	#install file system and utilities
	unzip -o "$ARCHIVEDIR/$ARCHIVE" -d $BIN_DIRECTORY

	#make sure the files got installed properly
	if [ ! -e "$BIN_DIRECTORY/MacFile" ]; then
		msg="Installation failed, configuration app failed to install!!"
		alert --stop "$msg" "Quit"
		exit 0
	fi

	if [ ! -e "$BIN_DIRECTORY/afp_server" ]; then
		msg="Installation failed, afp_server failed to install!!"
		alert --stop "$msg" "Quit"
		exit 0
	fi

	#creat links in the right places for easy access
	ln -s  -f $BIN_DIRECTORY/MacFile $LINK_PREF_DIRECTORY/MacFile
	ln -s  -f $BIN_DIRECTORY/afp_server $LINK_APPL_DIRECTORY/afp_server

	#create the mac utility volume folder
	mkdir $MACUTILITY_VOLUME	
	cp $CLRTXT_SCRIPT_NAME $MACUTILITY_VOLUME
	cp $README_NAME $MACUTILITY_VOLUME
	
	#install the create afp share tracker add-on
	cp $CREATESHARE_ADDON $TRACKER_ADDON_DIRECTORY
}

function removeFiles {
	#remove deprecated configuration tool
	rm -fv $BIN_DIRECTORY/AFPConfig
	rm -fv $LINK_PREF_DIRECTORY/AFPConfig
	rm -fv $BIN_DIRECTORY/MacFile
	rm -fv $LINK_PREF_DIRECTORY/MacFile

	#remove old afp_server application
	rm -fv $BIN_DIRECTORY/afp_server
	rm -fv $LINK_APPL_DIRECTORY/afp_server
	
	#remove the new style tracker add-on
	rm -fv $TRACKER_ADDON_DIRECTORY/$CREATESHARE_ADDON

	#remove deprecated tracker add-ons
	rm -fv "/boot/home/config/add-ons/Tracker/AFP-Create Share"
	rm -fv "/boot/home/config/add-ons/Tracker/AFP-Remove Share"
	rm -fv "/boot/home/config/add-ons/Tracker/AFP-Set Login Message"

	#the old BeUAM_Volume is deprecated as of v1.6.1 due to Apple
	#dropping support for UAM modules. Stop sharing the volume and
	#delete the files we put there. We leave the folder in case
	#someone put something else in there.
	rm -frv $UAM_DIRECTORY/SG-UAMPackage.pkg
	rm -fv $UAM_DIRECTORY/$CLRTXT_SCRIPT_NAME
	
	#remove the mac readme and clrtxt script files
	rm -fv $MACUTILITY_VOLUME/$README_NAME
	rm -fv $MACUTILITY_VOLUME//$CLRTXT_SCRIPT_NAME
}

function uninstall {
	result=$(alert --stop "Are you sure you want to completely remove MacFile Server for Haiku?" "Cancel" "Remove")
	
	if [ "$result" = "Remove" ]; then
		#Shutdown the current MacFile (afp_server) process
		echo shutting down the MacFile server...
		quit application/x-vnd.afp_server
	
		removeFiles
		
		(alert --info "The MacFile Server for Haiku has been removed. Click OK to exit." "OK")
		exit 0
	fi
	
	(alert --info "Removal of the MacFile Server has been cancelled leaving your installation untouched. Click OK to exit." "OK")
	exit 0
}

#The following function installs the commands necessary into the UserBootscript
#located in the /boot/home/config/boot directory. If a UserBootscript already
#exists, then we'll just append our boot script to it. Otherwise, we'll append
#our script to the UserBootscript.sample and rename it to UserBootscrip
#
#The script is necessary for the afp_server to launch at boot time automatically

BOOT_SCRIPT_FOLDER="/boot/home/config/boot"
BOOT_FILE_NAME="UserBootscript"
AFP_SIG="AFP_SIGNATURE_V1"
AFP_BOOTSCRIPT="afp_bootscript.sh"

function updatebootscript {
	#If there is already a UserBootscript, then rename it to .old
	if [ -e "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME" ]; then
		msg="You already have a UserBootscript configured. Do you want to append the commands necessary to autolaunch the MacFile Server at boot time?"
		result=$(alert --info "$msg" "NO" "YES")
		if [ "$result" = "YES" ]; then
			
			#Only append the boot script stuff if it is not already there
			result=$(grep "$AFP_SIG" "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME")
			if [ "$result" = "#$AFP_SIG" ]; then
				alert --info $"UserBootscrip already contains necessary commands."
			else
				#renaming old boot file
				mv "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME" "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME.old"
				
				#merging old and new boot scripts
				cat "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME.old" "$AFP_BOOTSCRIPT" > "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME"
			fi
		fi
	else
		cat "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME.sample" "$AFP_BOOTSCRIPT" > "$BOOT_SCRIPT_FOLDER/$BOOT_FILE_NAME"
	fi
}


#Make sure the zip file with the app bits is present
if [ ! -e "$ARCHIVEDIR/$ARCHIVE" ]; then
	msg="The file \"$ARCHIVE\" that contains the application binaries is missing. Installation aborted."
	alert --stop "$msg" "Quit"
	exit 0
fi

#We must shut down afp_server before installing the new one.
msg="If you already have MacFile installed, the current server will be shut down and connected users will be disconnected. Do you wish to continue?"
result=$(alert --stop "$msg" "Uninstall" "Cancel" "Install")
if [ "$result" = "Cancel" ]; then
	exit 0
fi

#If the user wants to uninstall, make sure, then do it.
if [ "$result" = "Uninstall" ]; then
	uninstall
fi

#Shutdown the current MacFile (afp_server) process
echo shutting down the MacFile server...
quit application/x-vnd.afp_server

removeFiles
installFiles
updatebootscript

#start the server
start $BIN_DIRECTORY/afp_server
waitfor afp_server
echo afp_server has been started

#share the uam volume
"$ARCHIVEDIR/share_volume"

result=$(alert --info $'MacFile has been installed and is now running. Would you like to open the configuration tool?' "No" "Yes")
if [ "$result" = "Yes" ]; then
	start $BIN_DIRECTORY/MacFile
fi

result=$(alert --info $'MacFile has been successfully installed.\nPlease read the release notes for further information.' "Open Release Notes" "Done")
if [ "$result" = "Open Release Notes" ] ; then	
	/boot/apps/BeZillaBrowser/BeZillaBrowser "$ARCHIVEDIR/ReleaseNotes.html"
fi
